#include <unistd.h>
#include <linux/sonypi.h>
#include <stdlib.h>

#include "ivad.h"
#include "sonypi_ctl.h"
#include "audio_ctl.h"
#include "event_handler.h"


const struct
{
	unsigned char event;	// SONYPI_EVENT_*
	unsigned event_type;	// EVENT_TYPE_*
	void *data;		// data or EVENT_FLAG_* or EVENT_MODE_*
	unsigned scope;		// EVENT_MODE_* (&&)
} event_mapping[] = { EVENT_MAPPING };


void handle_event(unsigned char event, bool private_event)
{
	int i;
	bool found_mapping = false;
	static __u8 current_mode = EVENT_MODE_NONE;

	if (private_event)
	{
		// check for timeout first because it happens very often
		if ( event == PRIVATE_EVENT_TIMEOUT)
		{
			current_mode = EVENT_MODE_NONE;
			return;
		}
	}
	
	dprint1("incoming event from %d", private_event);
	dprint1(" with event %d...\n", event);

	for (i = 0; event_mapping[i].event != EVENT_TYPE_NONE && !found_mapping; i++)
	{
		if (event == event_mapping[i].event &&
			(current_mode & event_mapping[i].scope) &&
			(!private_event || (private_event && (event_mapping[i].scope & EVENT_MODE_PRIVATE))))
		{
			switch (event_mapping[i].event_type)
			{
				case EVENT_TYPE_EXIT:
					dprint0("exiting...\n");
					_exit(0);
				case EVENT_TYPE_CHANGE_MODE:
					current_mode = (__u32)event_mapping[i].data;
					dprint1("mode change to %d\n", current_mode);
					break;
				case EVENT_TYPE_COMMAND:
					dprint0("executing command...\n");
					system((const char*)event_mapping[i].data);
					break;
				case EVENT_TYPE_BRIGHTNESS:
					switch ((__u32)event_mapping[i].data)
					{
						case EVENT_FLAG_INCREASE:
							set_brightness_relative(ivad_config.brightness_step);
							break;
						case EVENT_FLAG_DECREASE:
							set_brightness_relative(-ivad_config.brightness_step);
					}
					break;
				case EVENT_TYPE_BRIGHTNESS_SET:
					dprint1("setting brightness to %d\n", (__u32)event_mapping[i].data);
					set_brightness((__u32)event_mapping[i].data);
					break;
				case EVENT_TYPE_VOLUME:
					switch ((__u32)event_mapping[i].data)
					{
						case EVENT_FLAG_INCREASE:
							set_volume_relative(ivad_config.mixer_step);
							break;
						case EVENT_FLAG_DECREASE:
							set_volume_relative(-ivad_config.mixer_step);
					}
					break;
				case EVENT_TYPE_VOLUME_SET:
					set_volume((int)event_mapping[i].data);
					break;
				case EVENT_TYPE_MUTE:
					switch ((__u32)event_mapping[i].data)
					{
						case EVENT_FLAG_TOGGLE:
							dprint0("mute on/off\n");
							mute_toggle();
							break;
						case EVENT_FLAG_ON:
							dprint0("mute\n");
							mute(true);
							break;
						case EVENT_FLAG_OFF:
							dprint0("unmute\n");
							mute(false);
					}
					break;
				case EVENT_TYPE_BLUETOOTH:
					switch ((__u32)event_mapping[i].data)
					{
						case EVENT_FLAG_TOGGLE:
							dprint0("bluetooth on/off (not tested)\n");
							toggle_bluetooth();
							break;
						case EVENT_FLAG_ON:
							dprint0("bluetooth on (not tested)\n");
							set_bluetooth(1);
							break;
						case EVENT_FLAG_OFF:
							dprint0("bluetooth off (not tested)\n");
							set_bluetooth(0);
					}
					break;
				case EVENT_TYPE_LCD:
					switch ((__u32)event_mapping[i].data)
					{
						case EVENT_FLAG_TOGGLE:
							dprint0("lcd on/off (not implemented)\n");
							break;
						case EVENT_FLAG_ON:
							dprint0("lcd (not implemented)\n");
							break;
						case EVENT_FLAG_OFF:
							dprint0("external monitor (not implemented)\n");
					}
					break;
				case EVENT_TYPE_SUSPEND:
					dprint0("suspending... (not implemented)\n");
					break;
#ifdef DEBUG
				default:
					dprint1("error: event %d not found\n", event_mapping[i].event_type);
#endif
			}
			found_mapping = true;
		}
	}
	if (!found_mapping && !private_event)
		current_mode = EVENT_MODE_NONE;
}
