#include <sys/types.h>
#include <sys/ioctl.h>
#include <linux/sonypi.h>

#include "ivad.h"
#include "ivad_ioctl.h"
#include "sonypi_ctl.h"

//////////////////////////////////////////////////////////////////
// brightness
//////////////////////////////////////////////////////////////////

// returns the current brightness or a value < 0 on error
__s16 get_brightness()
{
	__u8 value;
	if (ivd_ioctl(SONYPI_IOCGBRT, &value) < 0)
		return -1;
	return value;
}

// returns -1 on error, 0 on success
__s8 set_brightness(__u8 value)
{
	if (ivd_ioctl(SONYPI_IOCSBRT, &value) < 0)
		return -1;
	return 0;	
}

void set_brightness_relative(__s16 value)
{
	int br;
	__s16 new_br;
	
	if ((br = get_brightness()) < 0)
		return;
	// repeat this due to a broken bios or kernel driver
	if ((br = get_brightness()) < 0)
		return;

	new_br = br + value;
	new_br = (new_br > 255) ? 0xff : (new_br < 0) ? 0 : new_br;

	if (new_br != br)
	{
		dprint1("brightness: %d\n", new_br);
		set_brightness(new_br);
	}
}


//////////////////////////////////////////////////////////////////
// bluetooth
//////////////////////////////////////////////////////////////////

// returns -1 on error, 0 on success
__s8 set_bluetooth(__u8 state)
{
	if (ivd_ioctl(SONYPI_IOCSBLUE, &state) < 0)
		return -1;
	return 0;	
}

// returns the current brightness or a value < 0 on error
__s16 get_bluetooth()
{
	__u8 state;
	if (ivd_ioctl(SONYPI_IOCGBLUE, &state) < 0)
		return -1;
	return state;
}

void toggle_bluetooth()
{
	__s16 current_state = get_bluetooth();
	if (current_state > 0)
	{
		dprint0("bluetooth: 0\n");
		set_bluetooth(0);
	}
	else if (!current_state)
	{
		dprint0("bluetooth: 1\n");
		set_bluetooth(1);
	}
}




//////////////////////////////////////////////////////////////////
// battery
//////////////////////////////////////////////////////////////////

// returns one of the following:
// 0: error
// SONYPI_BFLAGS_B1, SONYPI_BFLAGS_B2, SONYPI_BFLAGS_AC
__u8 get_battery_flags()
{
	__u8 flags, flags2, counter = 0;
	// useless repetition due to broken bios/driver
	do {
		flags2 = flags;
		if (ivd_ioctl(SONYPI_IOCGBATFLAGS, &flags) < 0)
			return 0;
	}
	while(flags != flags2 && counter++ < 5);
	return flags;	
}


__u8 battery1_is_connected()
{
	return(get_battery_flags() & SONYPI_BFLAGS_B1);
}

__u8 battery2_is_connected()
{
	return(get_battery_flags() & SONYPI_BFLAGS_B2);
}

__u8 ac_is_connected()
{
	return(get_battery_flags() & SONYPI_BFLAGS_AC);
}

__u16 get_battery_capacity(int event)
{
	__u16 capacity, capacity2, counter = 0;
	// useless repetition due to broken bios/driver
	do {
		capacity2 = capacity;
		if (ivd_ioctl(event, &capacity) < 0)
			return 0;
	} while (capacity2 != capacity && counter++ < 5);
	return capacity;
}

double get_battery_percent(int battery_nr)
{
	__u16 cap, rem;
	switch (battery_nr)
	{
		case 1:
			if (!battery1_is_connected())
				return(0.0);
			cap = get_battery_capacity(SONYPI_IOCGBAT1CAP);
			rem = get_battery_capacity(SONYPI_IOCGBAT1REM);
			break;
		case 2:
			if (!battery2_is_connected())
				return(0.0);
			cap = get_battery_capacity(SONYPI_IOCGBAT2CAP);
			rem = get_battery_capacity(SONYPI_IOCGBAT2REM);
			break;
		default:
			return(0.0);
	}
	
	if (cap == 0)
		return(0.0);
	
	return((double)rem / (double)cap);
}

